<?

/*

Dan's PHP Library (DPLib) v0.1

Copyright (C) 2000 Dan Steinman
Distributed under the terms of the GNU General Public License

dan@dansteinman.com
www.dansteinman.com

*/

### HTML/XML Parsing Functions

// formats a string with html for easier parsing
function html_formatTags($html) {
	$html = ereg_replace("( +)"," ",$html);
	$html = ereg_replace("( *)=( *)","=",$html);
	$html = ereg_replace("( *)<( *)","<",$html);
	$html = ereg_replace("( *)>( *)",">",$html);
	$html = ereg_replace("</( *)","</",$html);
	$html = ereg_replace("'","\"",$html);
	
	$chunks = split(" ",$html);
	$html = "";
	for ($i=0;$i<count($chunks);$i++) {
		$chunk = $chunks[$i];
		if (ereg("<",$chunk)) {
			$lchunk = strtolower($chunk);
			$html .= "$lchunk ";
		}
		else if (ereg("=",$chunk)) {
			if (!ereg("=\"",$chunk)) {  // adds quotes for unquoted values
				$chunk = ereg_replace("=","=\"",$chunk);
				$chunk .= "\"";
			}
			$tmp = split("=",$chunk);
			$name = strtolower($tmp[0]); // lowercase the attribute
			$name = chop($name); // trim excess whitespace
			$name = ltrim($name);
			$value = $tmp[1]; // dont lowercase the value
			$html .= $name."=".$value." ";
		}
		else {
			$html .= "$chunk ";
		}
	}
	
	return $html;
}

// returns array of all the attributes of a single html tag
// requires formatted tags
function html_parseTagAttributes($tag) {
	$html = ereg_replace("(.*)<","",$tag); // strip outter text
	$html = ereg_replace(">(.*)","",$tag); // strip inner text
	
	$attributes = array();

	$realchunks = array();
	$lookingforend = false;
	
	$chunks = split(" ",$html);
	
	for ($i=1;$i<count($chunks);$i++) {  // skip first "<tag" chunk
		$chunk = $chunks[$i];
		if ($lookingforend) {
			if (ereg("=",$chunk)) {
				$lookingforend = false;
			}
			else {
				$last = count($realchunks)-1;
				$realchunks[$last] .= "$chunk";
				
				if ( ereg("=\"(.*)\"",$realchunks[$last]) ) {  // found last piece: value"
					$lookingforend = false;
				}
				else {
					$realchunks[$last] .= " ";
				}
				continue;
			}
		}
		if (ereg("=",$chunk)) {
			$realchunks[] = "$chunk";
			
			if (!ereg("=\"(.*)\"",$chunk)) {  // is not complete: "value
				$last = count($realchunks)-1;
				$realchunks[$last] .= " ";
				$lookingforend = true;
			}
		}
		else {
			// extraneous attribute with no value eg. <table nowrap>
		}
	}
	
	for ($i=0;$i<count($realchunks);$i++) {
		$chunk = $realchunks[$i];
		$chunk = ereg_replace("\"","",$chunk); // strip quotes
		$parts = split("=",$chunk,2);
		$name = $parts[0];
		$value = $parts[1];
		
		$attributes[$name] = $value;
	}
	
	return $attributes;
}

// parse out the forms and all their data from the page and return as an array
// requires formatted tags

// FIX SELECT,TEXTAREA,RADIO

function html_parseForms($html) {
	// split html at each tag
	$code = ereg_replace("\n","",$html);
	$code = ereg_replace("<","\n<",$html);
	$tags = split("\n",$code);
	
	if (!strpos($tags[0],"<")) {  // strip http data
		$tags = ar_shift($tags);
	}
	
	$forms = array();
	$inform = false;
	
	for ($i=0;$i<count($tags);$i++) {
		$tag = $tags[$i];

		if (!$inform && ereg("<form",$tag)) {
			$att = html_parseTagAttributes($tag);
			$action = $att["action"];
			// note: $action will not contain the full url if it was not specified
			
			$method = $att["method"];
			if (!$method) $method = "get";
			
			$forms[] = array(
				"action" => $action,
				"method" => $method,
				"inputs" => array()
			);

			$inform = true;
		}
		if ($inform) {
			if (ereg("<input",$tag)) {
				$att = html_parseTagAttributes($tag);
				$name = $att["name"];
				if ($name) {
					$type = $att["type"];
					$value = $att["value"];
					$forms[count($forms)-1]["inputs"][] = array($name,$type,$value);
				}
			}
			else if (eregi("</form",$tags[$i],$regs)) {
				$inform = false;
			}
		}
		
	}
	
	return $forms;
}

function html_getPrintableHTML($string) {
	$string = ereg_replace("<","&lt;",$string);
	$string = ereg_replace("<","&lt;",$string);
	$string = ereg_replace(" ","&nbsp;",$string);
	return $string;
}

### Socket Functions

// return content of a url as a string
function http_getFile($url) {
	$server = ereg_replace("http://","",$url);
	$file = strstr($server,"/");
	$server = ereg_replace("$file","",$server);
	$port = 80;
	$fp = fsockopen($server, $port, &$errno, &$errstr);
	if ($fp) {
		fputs($fp,"GET http://$server$file HTTP/1.0\nHost: $server\n\n");
		$lines = array();
		while ($line = fgets($fp, 4096)) {
			$lines[] = $line;
		}
		fclose($fp);
		$html = implode($lines,"\n");
	}

	if ($html == "" || !$fp) {  // try fopen()
		$fp = fopen($url, "r");
		if ($fp) {
			$lines = array();
			while ($line = fgets($fp, 4096)) {
				$lines[] = $line;
			}
			fclose($fp);
			$html = implode($lines,"\n");
		}
	}
	return $html;
}

### Array Functions

// note: these are not necessary if you use PHP 4.0

// returns the index of a value
function ar_indexOf($array,$val) {
	for ($i=0;$i<count($array);$i++) {
		if ($array[$i] == $val) {
			return $i;
		}
	}
	return -1;
}

// removes the element of the given index
function ar_remove($array,$index) {
	$temp = array();
	for ($i=0;$i<$index;$i++) {
		$temp[$i] = $array[$i];
	}
	for ($i=$index;$i<count($array)-1;$i++) {
		$temp[$i] = $array[$i+1];
	}
	return $temp;
}

// inserts the element into the given index
function ar_insert($array,$index,$val) {
	$temp = array();
	for ($i=0;$i<$index;$i++) {
		$temp[$i] = $array[$i];
	}
	$temp[$index] = $val;
	for ($i=$index+1;$i<count($array)+1;$i++) {
		$temp[$i] = $array[$i-1];
	}
	return $temp;
}

// replaces the element of the given $index with the new $val sent
function ar_replace($array,$index,$val) {
	$l = count($array);
	$array = ar_removeAt($array,$l);
	$array = ar_insertAt($array,$l,$val);
	return $array;
}

// removes the last element
function ar_pop($array) {
	return ar_remove($array,count($array)-1);
}
// pop an element of the beginning of array and returns the new array
function ar_shift($array) {
	return ar_remove($array,0);
}

// inserts as the last element
function ar_push($array,$val) {
	$array[] = $val;
	return $array;
}

// push one or more elements onto the beginning of array and returns the new array
function ar_unshift($array,$val) {
	return ar_insert($array,0,$val);
}


// prints all the values separated by \n
function ar_printValues($array) {
	ar_printValuesWith($array,"\n");
}

// prints all the values separated a given separator
function ar_printValuesWith($array,$sep) {
	for ($i=0;$i<count($array);$i++) {
		echo "$array[$i]";
		echo "$sep";
	}
}

### Generic Utility Functions

function debug($error) {
	print "<br clear=all><b>Error:</b> <i>$error</i>";
	die();
}


### Database Connection Classes

// note: it really only does MySQL -- anyone want to do others?

class MySQLDatabase {
	var $host = "";
	var $username = "";
	var $password = "";
	var $database = "";
	
	var $connected = false;
	
	function setHost($host) {
		$this->host = $host;
	}
	function setUsername($username) {
		$this->username = $username;
	}
	function setPassword($password) {
		$this->password = $password;
	}
	function setPort($port) {
		$this->port = $port;
	}
	
	function connect() {
		$this->link = mysql_connect($this->host,$this->username,$this->password);
		if ($this->link) {
			$this->connected = true;
			return $this->link;
		}
		else {
			debug("unable to connect to database");
		}
	}

	function select_db($database) {
		$this->database = $database;
		if (!$this->connected) $this->connect();
		if (mysql_select_db($database,$this->link)) return true;
		else {
			debug("could not select database $database");
		}
	}

	function query($query) {
		return mysql_query($query,$this->link);
	}
	function num_rows($result) {
		return mysql_num_rows($result);
	}
	function fetch_array($result) {
		return mysql_fetch_array($result);
	}
	function fetch_row($result) {
		return mysql_fetch_row($result);
	}
	
	function checkPassword($username, $password, $table) {
		if ($table==null) $table = "users";
		$result = mysql_query("select * from $table where(username='$username' && password=password('$password'))");
		if ($result && $this->num_rows($result)==1) return true;
		else {
			debug("bad username/password");
		}
	
	}

}
